/* $Id: hellosign.c,v 1.18 2011/02/24 13:19:11 rjs Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * hellosign sample: generate a document in memory with PDFlib, and sign it
 * with PLOP
 */

#include <stdio.h>

#include "pdflib.h"
#include "ploplib.h"

int
main(void)
{
    PDF *p;
    PLOP *plop;
    int font;
    const char *contents = NULL;
    long size;

    /* This is where input files live. Adjust as necessary. */
    char *searchpath = "../data";

    const char *out_filename = "hellosign.pdf";

    static const char sign_opts[] = "sign={"
	    "appearance={fieldname=Signature1 } "
#if defined(USE_MSCAPI)
	    "engine=mscapi "
	    "digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
#elif defined(USE_PKCS11)
	    "engine=pkcs#11 "
	    "digitalid={filename=cryptoki.dll}"
#else
	    "digitalid={filename=demo2048.p12} "
#endif
	    "password={demo} "
	    "reason={I agree to this document} "
	    "contactinfo={tel (123) 456 789} "
	    "location=Munich "
	    "}";

    if ((p = PDF_new()) == (PDF *) 0)
    {
        printf("Couldn't create PDFlib object (out of memory)!\n");
        return(98);
    }

    /* ------------- Step 1: generate the document in memory ------------ */

    PDF_TRY(p) {
	/* open new PDF file in memory */

	PDF_begin_document(p, "", 0, "");

	PDF_set_info(p, "Creator", "hellosign.c");
	PDF_set_info(p, "Author", "Rainer Plckl");
	PDF_set_info(p, "Title", "Hello, world/signed (C)!");

	font = PDF_load_font(p, "Helvetica-Bold", 0, "host", "");

	PDF_begin_page_ext(p, a4_width, a4_height, "");

	PDF_setfont(p, font, 24);
	PDF_set_text_pos(p, 50, 700);
	PDF_show(p, "Hello, world/signed!");
	PDF_continue_text(p, "(says C)");
	PDF_end_page_ext(p, "");

	PDF_end_document(p, "");

	/* fetch the buffer containing the PDF document */
	contents = PDF_get_buffer(p, &size);
    }

    PDF_CATCH(p) {
        printf("PDFlib exception occurred in hello sample:\n");
        printf("[%d] %s: %s\n",
            PDF_get_errnum(p), PDF_get_apiname(p), PDF_get_errmsg(p));
        PDF_delete(p);
        return(98);
    }

    /* ------------------ Step 2: sign the document ----------------- */
	
    /* create a new PLOP object */
    if ((plop = PLOP_new()) == (PLOP *) 0)
    {
	printf("Couldn't create PLOP object (out of memory)!\n");
	return(99);
    }

    PLOP_TRY(plop) {
	int doc;

	char optlist[1024];

	sprintf(optlist, "searchpath={%s}", searchpath);
	PLOP_set_option(plop, optlist);

	/* Create a virtual PVF file from the data in memory */
	PLOP_create_pvf(plop, "/pvf/hello.pdf", 0, contents, size, "");

	/* open unsigned input document from PVF file */
	if ((doc = PLOP_open_document(plop, "/pvf/hello.pdf", 0, "")) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    PDF_delete(p);
	    return(99);
	}

	/* sign the output file */
	if (PLOP_create_file(plop, out_filename, 0, sign_opts) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    PDF_delete(p);
	    return(99);
	}

	/* close the output file */
	PLOP_close_document(plop, doc);

	/* delete PVF file containing input document */
	if (PLOP_delete_pvf(plop, "/pvf/hello.pdf", 0) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    PDF_delete(p);
	    return(99);
	}
    }

    PLOP_CATCH(plop) {
	printf("PLOP exception occurred in hellosign:\n");
	printf("[%d] %s: %s\n", PLOP_get_errnum(plop),
		PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	PLOP_delete(plop);
	return(99);
    }

    /* delete the PLOP object */
    PLOP_delete(plop);

    /* delete the PDF object */
    PDF_delete(p);

    return 0;
}
